//=============================================================================
// CompatibilitySave&Load.js
//=============================================================================
/*:
 * @plugindesc Makes changes to the Improved Save & Load Menu mod if it exists. For use in a playtesting enviroment, remove "if ($modLoader.knownMods.has("saveloadplus"))" from line 78 and the closing bracket in 153.
 *
 * @author Pyro#3607
 *
 * @help
 *
 * Makes changes to the Improved Save & Load Menu mod if it exists
 *
 * TERMS OF USE
 * This plugin is licensed under the "Do What the Fuck You Want To Public License". Use it how you see fit. Do credit me if you want, though.
 *
 * COMPATIBILITY
 * There shouldn't be any issues. Load after the original plugin. Will do nothing if you don't have the mod installed.
 */
(function() {

      //=============================================================================
      // * Change Save Face Image
      //=============================================================================
      // USAGE INSTRUCTIONS: JUST ADD ANOTHER CASE AND MAKE IT RETURN THE FACE SPRITE WITHOUT AN EXTENSION.

    Game_Actor.prototype.faceSaveLoad = function() {


        var actor = this.actor();
        // When changing these the .png should not be required.
        switch (actor.id) {
        case 1: // Omori
        return "01_OMORI_BATTLE";
        case 2: // Aubrey
        return "02_AUBREY_BATTLE";
        case 3: // Kel
        return "03_KEL_BATTLE";
        case 4: // Hero
        return "04_HERO_BATTLE";
        case 8: // Omori
        return "01_FA_OMORI_BATTLE";
        case 9: // Aubrey
        return "02_FA_AUBREY_BATTLE";
        case 10: // Kel
        return "03_FA_KEL_BATTLE";
        case 11: // Hero
        return "04_FA_HERO_BATTLE";
        default:
            return "01_OMORI_BATTLE"; // if ther is one?
        }
    };

      //=============================================================================
      // * Change Save Face Index
      //=============================================================================
      // USAGE INSTRUCTIONS: AS FAR AS I KNOW, THIS CHANGES THE INDEX OF THE IMAGE. 
      //                     IN OTHER WORDS, IF YOU WANT TO, FOR EXAMPLE, USE OMORI'S HAPPY SPRITE INSTEAD OF NEUTRAL, MAKE IT RETURN 1.

    Game_Actor.prototype.faceSaveLoadIndex = function() {
        var actor = this.actor();
        // When changing these the .png should not be required.
        switch (actor.id) {
          case 1: // Omori
          return 0;
          case 2: // Aubrey
          return 0;
          case 3: // Kel
          return 0;
          case 4: // Hero
          return 0;
          default:
            return 0;
        }
      };

      //=============================================================================
      // * Change Save Background
      //=============================================================================
      // USAGE INSTRUCTIONS: DON'T CHANGE THIS. EVER. UNLESS YOU KNOW WHAT YOU'RE DOING AND THINK YOU'RE BETTER AT THIS THAN ME (you probably are)
      
      if (Utils.isOptionValid('test')){
        Window_OmoriFileInformation.prototype.refresh = function(valid, info, id) {
          // Clear Contents
          this.contents.clear();
          // Get Color
          var color = 'rgba(255, 255, 255, 1)';
          // Get ID
          //var id = this._index + 1;
          //var valid = DataManager.isThisGameFile(id);
          //var info = DataManager.loadSavefileInfo(id);
        
          // Draw Lines
          this.contents.fillRect(0, 29, this.contents.wi0dth, 3, color);
        
          for (var i = 0; i < 3; i++) {
            var y = 55 + (i * 25)
            this.contents.fillRect(113, y, this.contents.width - 117, 1, color);
          };
        
        
          // Draw File
          this.contents.fontSize = 30;
          if(this._columns === 1)this.contents.drawText('FILE ' + id + ':', 10 + 30, -5, 100, this.contents.fontSize);
          else{
            if(!valid || info.saveName === null || info.saveName === "" ||info.saveName === undefined)this.contents.drawText(id, 2, -5, 100, this.contents.fontSize, 'center');
            else this.contents.drawText(info.saveName, 2, -5, 98, this.contents.fontSize, 'center');
          }
          
          // If Valid
          if (valid) {
            if(this._select){
              this._faceSprite.visible = true;
              this._selectedBG.visible = true;
            }else{
              this._faceSprite.visible = false;
              this._selectedBG.visible = false;
            }
            this.contents.drawText(info.chapter, 85 + 30 + 13*Math.floor(Math.log10(id)), -5, this.contents.width, this.contents.fontSize);
        
            this.contents.fontSize = 28;
        
            let backBitmap = ImageManager.loadSystem('loadscreen_backgrounds');
            let width = 100;
            let height = 100;
            // this.contents.blt(backBitmap, 0, 0, width, height, 0, 34, width + 10, height);
            bgLocation = getBackground(info.location);
            this.contents.blt(backBitmap, width*bgLocation[0], height*bgLocation[1], width, height, 1, 33); //width*n, height*m controls background
            this._selectedBG.setFrame(width*bgLocation[0], height*bgLocation[1], width, height);
            // Get Actor
            var actor = info.actorData;
            // Draw Actor Face
            this.drawFace(actor.faceName, 0, -2, this.contents.height - Window_Base._faceHeight + 7, Window_Base._faceWidth, height - 2);
            this._faceSprite.actor = actor;
            if(omoDelete)this._faceSprite.setAnimRow(3);
            else this._faceSprite.setAnimRow(0);
            // Draw Actor Name
            this.contents.fontSize = 24;
            if(info.saveName === null || info.saveName === "" || info.saveName === undefined)this.contents.drawText(actor.name, 118, 30, 200, 24);
            else this.contents.drawText(info.saveName, 118, 30, 200, 24);
            // Draw Level
            this.contents.drawText('LEVEL:', 290 + 55, 30, 100, 24);
            this.contents.drawText(actor.level, 290 + 55, 30, 70, 24, 'right');
            // Draw Total PlayTime
            this.contents.drawText('TOTAL PLAYTIME:', 118, 55, 200, 24);
            this.contents.drawText(info.playtime, 295 + 55, 55, 100, 24);
            // Draw Location
            this.contents.drawText('LOCATION:', 118, 80, 200, 24);
            this.contents.drawText(info.location, 205, 80, 210, 24, 'right');
          }else{
            this._faceSprite.visible = false;
            this._selectedBG.visible = false;
          };
        
          // Draw Border
          this.contents.fillRect(102, 32, 3, 102, 'rgba(255, 255, 255, 1)')
          this.contents.fillRect(0, 29, 108, 3, 'rgba(255, 255, 255, 1)')
        };
        
      } else{
        
      if ($modLoader.knownMods.has("saveloadplus")){
      Window_OmoriFileInformation.prototype.refresh = function(valid, info, id) {
        // Clear Contents
        this.contents.clear();
        // Get Color
        var color = 'rgba(255, 255, 255, 1)';
        // Get ID
        //var id = this._index + 1;
        //var valid = DataManager.isThisGameFile(id);
        //var info = DataManager.loadSavefileInfo(id);
      
        // Draw Lines
        this.contents.fillRect(0, 29, this.contents.width, 3, color);
      
        for (var i = 0; i < 3; i++) {
          var y = 55 + (i * 25)
          this.contents.fillRect(113, y, this.contents.width - 117, 1, color);
        };
      
      
        // Draw File
        this.contents.fontSize = 30;
        if(this._columns === 1)this.contents.drawText('FILE ' + id + ':', 10 + 30, -5, 100, this.contents.fontSize);
        else{
          if(!valid || info.saveName === null || info.saveName === "" ||info.saveName === undefined)this.contents.drawText(id, 2, -5, 100, this.contents.fontSize, 'center');
          else this.contents.drawText(info.saveName, 2, -5, 98, this.contents.fontSize, 'center');
        }
        
        // If Valid
        if (valid) {
          if(this._select){
            this._faceSprite.visible = true;
            this._selectedBG.visible = true;
          }else{
            this._faceSprite.visible = false;
            this._selectedBG.visible = false;
          }
          this.contents.drawText(info.chapter, 85 + 30 + 13*Math.floor(Math.log10(id)), -5, this.contents.width, this.contents.fontSize);
      
          this.contents.fontSize = 28;
      
          let backBitmap = ImageManager.loadSystem('loadscreen_backgrounds');
          let width = 100;
          let height = 100;
          // this.contents.blt(backBitmap, 0, 0, width, height, 0, 34, width + 10, height);
          bgLocation = getBackground(info.location);
          this.contents.blt(backBitmap, width*bgLocation[0], height*bgLocation[1], width, height, 1, 33); //width*n, height*m controls background
          this._selectedBG.setFrame(width*bgLocation[0], height*bgLocation[1], width, height);
          // Get Actor
          var actor = info.actorData;
          // Draw Actor Face
          this.drawFace(actor.faceName, 0, -2, this.contents.height - Window_Base._faceHeight + 7, Window_Base._faceWidth, height - 2);
          this._faceSprite.actor = actor;
          if(omoDelete)this._faceSprite.setAnimRow(3);
          else this._faceSprite.setAnimRow(0);
          // Draw Actor Name
          this.contents.fontSize = 24;
          if(info.saveName === null || info.saveName === "" || info.saveName === undefined)this.contents.drawText(actor.name, 118, 30, 200, 24);
          else this.contents.drawText(info.saveName, 118, 30, 200, 24);
          // Draw Level
          this.contents.drawText('LEVEL:', 290 + 55, 30, 100, 24);
          this.contents.drawText(actor.level, 290 + 55, 30, 70, 24, 'right');
          // Draw Total PlayTime
          this.contents.drawText('TOTAL PLAYTIME:', 118, 55, 200, 24);
          this.contents.drawText(info.playtime, 295 + 55, 55, 100, 24);
          // Draw Location
          this.contents.drawText('LOCATION:', 118, 80, 200, 24);
          this.contents.drawText(info.location, 205, 80, 210, 24, 'right');
        }else{
          this._faceSprite.visible = false;
          this._selectedBG.visible = false;
        };
      
        // Draw Border
        this.contents.fillRect(102, 32, 3, 102, 'rgba(255, 255, 255, 1)')
        this.contents.fillRect(0, 29, 108, 3, 'rgba(255, 255, 255, 1)')
      };
      } else {Window_OmoriFileInformation.prototype.refresh = function() {
        // Clear Contents
        this.contents.clear();
        // Get Color
        var color = 'rgba(255, 255, 255, 1)';
        // Get ID
        var id = this._index + 1;
        var valid = DataManager.isThisGameFile(id);
        var info = DataManager.loadSavefileInfo(id);
      
        // Draw Lines
        this.contents.fillRect(0, 29, this.contents.width, 3, color);
        for (var i = 0; i < 3; i++) {
          var y = 55 + (i * 25)
          this.contents.fillRect(113, y, this.contents.width - 117, 1, color);
        };
      
      
        // Draw File
        this.contents.fontSize = LanguageManager.getMessageData("XX_BLUE.Window_OmoriFileInformation").refresh_contents_fontsize;
        let loc_position = LanguageManager.getMessageData("XX_BLUE.Window_OmoriFileInformation").file_position
        this.contents.drawText(LanguageManager.getMessageData("XX_BLUE.Omori_Save_Load").file.format(id), loc_position[0], loc_position[1], 100, this.contents.fontSize);
        // If Valid
        if (valid) {
          loc_position = LanguageManager.getMessageData("XX_BLUE.Window_OmoriFileInformation").refresh_drawText_position;
          let chap = LanguageManager.getMessageData("XX_BLUE.Chapter_Names")[info.chapter]
          if(!chap) {
            chap = info.chapter
          }
          this.contents.drawText(chap, loc_position[0], loc_position[1], this.contents.width, this.contents.fontSize);
          this.contents.fontSize = 28;
      
          let backBitmap = ImageManager.loadSystem('faceset_states');
          let width = backBitmap.width / 4;
          let height = backBitmap.height / 5;
          // this.contents.blt(backBitmap, 0, 0, width, height, 0, 34, width + 10, height);
          this.contents.blt(backBitmap, 0, 0, width, height, 1, 33);
          // Get Actor
          var actor = info.actorData
          // Draw Actor Face
          let bit = ImageManager.loadFace(actor.faceName);
          bit.addLoadListener(() => this.drawFace(actor.faceName, actor.faceIndex, -2, this.contents.height - Window_Base._faceHeight + 7, Window_Base._faceWidth, height - 2));
          // Draw Actor Name
          this.contents.fontSize = 24;
          this.contents.drawText(actor.name, 118, 30, 100, 24);
          // Draw Level
          loc_position = LanguageManager.getMessageData("XX_BLUE.Window_OmoriFileInformation").level_position;
          this.contents.drawText(LanguageManager.getMessageData("XX_BLUE.Omori_Save_Load").level, loc_position[0], loc_position[1], 100, 24);
          this.contents.drawText(actor.level, loc_position[0], loc_position[1], 70, 24, 'right');
          // Draw Total PlayTime
          loc_position = LanguageManager.getMessageData("XX_BLUE.Window_OmoriFileInformation").playtime_position;
          this.contents.drawText(LanguageManager.getMessageData("XX_BLUE.Omori_Save_Load").playtime, 118, 55, 200, 24);
          this.contents.drawText(info.playtime, loc_position[0], loc_position[1], 100, 24);
          // Draw Location
          this.contents.drawText(LanguageManager.getMessageData("XX_BLUE.Omori_Save_Load").location, 118, 80, 200, 24);
          this.contents.drawText(info.location, 205, 80, 210, 24, 'right');
        };
      }
    }
  }

    
function getBackground(location){
    // USAGE INSTRUCTIONS: THIS CHANGES THE SAVE BACKGROUND.
    //                     IT PULLS THEM FROM img/system/loadscreen_background.png.
    //
    //                     Just add another case with the map's display name.
    //                     The number have to return the "tile" you want to use from loadscreen_background. 
    //                     First number is x, second is y. The values are 0 indexed.
  switch(location){
    case "ORANGE OASIS":
      return [1,0];
    case "BLACK SPACE":
      return [2,0];
    case "PINWHEEL FOREST":
      return [3,0];
    case "VAST FOREST":
      return [3,1];
    case "FOREST PLAYGROUND":
      return [0,2];
    case "OTHERWORLD":
      return [2,1];
    case "JUNKYARD":
      return [1,2];
    case "LAST RESORT":
      return [0,1];
    case "DEEPER WELL":
      return [1,1];
    case "HUMPHREY":
      return [2,2];
    case "RAIN TOWN":
      return [3,2];
    case "SNOWGLOBE MOUNTAIN":
      return [0,3];
    case "FROZEN LAKE":
      return [1,3];
    case "BACKSTAGE":
      return [2,3];
    case "SPROUT MOLE VILLAGE":
      return [3,3];
    case "LOST LIBRARY":
      return [0,4];
    case "SWEETHEART'S CASTLE":
      return [1,4];
    case "PYREFLY FOREST":
      return [2,4];
    case "UNDERWATER HIGHWAY":
      return [3,4];
    case "BOSS RUSH":
      return [4,0];
    case "BASIL'S HOUSE":
      return [4,1];
    case "FARAWAY PARK":
      return [4,2];
    case "OUTSIDE":
      return [4,3];
    case "RECYCULTIST'S HQ":
      return [4,4];
    case "MOM'S ROOM":
      return [0,5];
    case "LOST FOREST":
      return [1,5];//i still need pictures of this :<
    case "NEIGHBOR'S ROOM":
      return [2,5];//i still need pictures of this :<
      case "THE ABYSS":
        return [3,5];
    case "PRE-NIGHT STORE":
      return [0,6];
    case "CUSTOM NIGHT PIZZERIA":
      return [1,6];
    default:
      return [0,0];
  }
}
    //=============================================================================
    // * RESERVE IMAGE FACES
    //=============================================================================
    // USAGE INSTRUCTIONS: THIS EXISTS SO THE GAME PRELOADS THE FACE SPRITES. 
    //                     IF THIS DOESN'T EXIST, THEY'LL APPEAR BLANK UNTIL YOU HOVER OVER THEM.
    //
    //                     Just replace "01_OMORI_BATTLE" with whatever face sprite you put into FaceSaveLoad (the upmost function).
    //                     If you need more sprites, just paste more in.


    let old_loadReservedBitmaps = Scene_OmoriFile.prototype.loadReservedBitmaps;
    Scene_OmoriFile.prototype.loadReservedBitmaps = function() {
      old_loadReservedBitmaps.call(this);
      // Super Call
      Scene_Base.prototype.loadReservedBitmaps.call(this);
      // Go through face
      ImageManager.reserveFace("01_OMORI_BATTLE", 0, this._imageReservationId);
    };
})();